"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const fact_1 = require("./fact");
const utils_1 = require("../utils");
const md5Hex = require('md5-hex');
exports.ROOT_NAME = "bom";
function getRuleStructure(fact, rule, ruleIsArray, facts) {
    let code = rule.getRule();
    const vars = utils_1.extractUndeclaredVarsFromCode(code);
    vars.forEach((name) => {
        const factName = new RegExp(name + "(?=([^\"]*\"[^\"]*\")*[^\"]*$)", "gi");
        code = code.replace(factName, `${exports.ROOT_NAME}.${fact.getPath(name)}`).replace("bom.bom", "bom");
    });
    return {
        name: (fact.path || rule.name) + (ruleIsArray ? "[]" : ""),
        code: code,
        ruleCode: rule.name,
        behaviour: rule.behaviour,
        absolute: undefined,
        variables: rule.expectedFacts(fact.pathMapping),
        enumerations: fact.enumerations
    };
}
class Aggregate {
    constructor(parent, aggregate) {
        this.type = "Aggregate";
        this.id = aggregate.id;
        this.name = aggregate.name;
        this.description = aggregate.description;
        this.version = aggregate.version;
        this.facts = [];
        aggregate.facts && aggregate.facts.forEach((fact) => {
            this.facts.push(new fact_1.default(fact));
        });
        this.facts.forEach((fact) => {
            fact.expectedFacts = fact.expectedFacts.map((e) => {
                const fact2 = fact.pathMapping.find((f) => { return f.replaceName === e; });
                return fact2 && fact2.withPath || e;
            });
        });
        this.sampleBOMs = aggregate.sampleBOMs || [];
        this.passed = aggregate.passed || false;
    }
    getOutputs() {
        return this.facts.filter(fact => fact.rule || fact.rules);
    }
    getRules() {
        const outputs = this.getOutputs()
            .filter(fact => fact.rule || fact.rules), rules = [];
        outputs
            .forEach((fact) => {
            if (fact.rule) {
                rules.push(getRuleStructure(fact, fact.rule, false, this.facts));
            }
            else if (fact.rules) {
                fact.rules.forEach(rule => rules.push(getRuleStructure(fact, rule, true, this.facts)));
            }
        });
        return rules;
    }
    schemaVersion() {
        const data = this.facts.map(fact => fact.path + "." + (fact.rule && fact.rule.dataType)).join("|");
        return md5Hex(data);
    }
    missingExpectedFacts() {
        let missing = [];
        this.facts.forEach((fact) => {
            const expectedFacts = fact.expectedFacts.map(name => fact.getPath(name));
            if (expectedFacts !== undefined) {
                expectedFacts.forEach((expected) => {
                    let missingEntry = missing.find(entry => entry.name === expected);
                    if (missingEntry === undefined) {
                        missingEntry = {
                            name: expected,
                            expectedBy: []
                        };
                        missing.push(missingEntry);
                    }
                    missingEntry.expectedBy.push(fact.path || (fact.rule && fact.rule.name) || "");
                });
            }
        });
        return missing.filter((entry) => {
            const factName = new RegExp(entry.name, "gi");
            return this.facts.find((fact) => {
                return fact.path && fact.path.match(factName);
            }) === undefined;
        });
    }
    sampleJson() {
        const sampleBom = {};
        this.missingExpectedFacts().forEach((entry) => {
            utils_1.addValueToJsonPath(sampleBom, `${exports.ROOT_NAME}.${entry.name}`, "missing");
        });
        this.facts.forEach((fact) => {
            const path = `${exports.ROOT_NAME}.${fact.path}`;
            if (fact.rule && fact.rule.statedAs === undefined) {
                let value = fact.rule.sampleValue !== undefined ? fact.rule.sampleValue : utils_1.sampleValue(fact.rule.dataType);
                utils_1.addValueToJsonPath(sampleBom, path, value);
            }
            else if (fact.rule === undefined) {
                if (fact.dataType === "string") {
                    utils_1.addValueToJsonPath(sampleBom, path, fact.sampleValue);
                }
                else if ((fact.dataType === "number")) {
                    utils_1.addValueToJsonPath(sampleBom, path, 10);
                }
                else if (fact.dataType === "date") {
                    utils_1.addValueToJsonPath(sampleBom, path, new Date().getTime());
                }
                else if (fact.dataType === "boolean") {
                    utils_1.addValueToJsonPath(sampleBom, path, false);
                }
                else {
                    utils_1.addValueToJsonPath(sampleBom, path, "Unknown");
                }
            }
        });
        return sampleBom;
    }
    sampleJsonWithDataTypes() {
        const sampleBom = {};
        this.missingExpectedFacts().forEach((entry) => {
            utils_1.addValueToJsonPath(sampleBom, `${exports.ROOT_NAME}.${entry.name}`, "unknown:0");
        });
        this.facts.forEach((fact) => {
            const path = `${exports.ROOT_NAME}.${fact.path}`;
            let value = (fact.rule && fact.rule.dataType) || "string";
            let id = (fact.rule && fact.rule.id) || 0;
            utils_1.addValueToJsonPath(sampleBom, path, `${value}:${id}`);
        });
        return sampleBom;
    }
}
exports.default = Aggregate;
//# sourceMappingURL=aggregate.js.map