"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const { Parser } = require("acorn");
const recast = require("recast");
exports.space = " ";
exports.commaSpace = ", ";
exports.comma = ",";
const detect = require('acorn-globals');
exports.specialKeywords = ["space", "comma", "commaSpace"];
exports.sampleValue = (dataType) => {
    if (dataType === "string") {
        return "Sample";
    }
    else if (dataType === "number") {
        return 100;
    }
    else if (dataType === "boolean") {
        return true;
    }
    else if (dataType === "date") {
        return new Date();
    }
    else {
        return undefined;
    }
};
exports.assert = (expr, message, code) => {
    if (!expr) {
        throw new Error(`[${code}]: ${message}`);
    }
    return exports.assert;
};
exports.getPath = (path) => {
    const paths = path.split(".").reverse();
    paths.pop();
    paths.reverse();
    return paths;
};
exports.addValueToJsonPath = (json, path, defaultValue, rootName) => {
    const root = json;
    path = path.replace(/\[\]/g, ".?");
    const paths = rootName ? exports.getPath(`${rootName}.${path}`) : exports.getPath(`${path}`);
    let parentNode = {};
    let child = "";
    paths.forEach((path) => {
        if (path !== "?") {
            parentNode = json;
            try {
                if (json[path] === undefined) {
                    json[path] = {};
                }
            }
            catch (e) {
                console.error(`Failed to add path in ${paths}: ${JSON.stringify(root, undefined, 2)}`);
                throw e;
            }
            json = json[path];
            child = path;
        }
        else {
            if (parentNode[child][0] === undefined) {
                parentNode[child] = [];
            }
            json = parentNode[child];
            parentNode = json;
            child = json.length;
        }
    });
    if (defaultValue !== undefined) {
        parentNode[child] = defaultValue;
        if (child === "?") {
            parentNode[child] = undefined;
        }
    }
    else {
        delete parentNode[child];
    }
    return root;
};
exports.assertValueForPath = (root, paths, testValue) => {
    return exports.getValueForPath(root, paths) + "" === testValue + "";
};
exports.findUsedBomVariablesInCode = (code, rootName) => {
    const bomVariablesRx = new RegExp(rootName + "\.([A-Za-z_])+([A-Za-z_0-9\.\[\]])*", "gi");
    const bomVariables = code.match(bomVariablesRx);
    const arrayExpressionsRx = /\[([A-Za-z0-9\+_'' \(\)\t\n\[\]])+\]/g;
    if (bomVariables === null) {
        return [];
    }
    for (let i = 0; i < bomVariables.length; i++) {
        bomVariables[i] = bomVariables[i].replace(arrayExpressionsRx, ".?");
    }
    return bomVariables.filter((variable, index, array) => {
        return array.lastIndexOf(variable) === index;
    });
};
const walk = (tree, captured, declared) => {
    if (tree.left) {
        walk(tree.left, captured, declared);
    }
    if (tree.right) {
        walk(tree.right, captured, declared);
    }
    else if (tree.type && (tree.type === "CallExpression")) {
        tree.arguments.forEach((arg) => {
            if ((arg.type === "FunctionExpression") ||
                (arg.type === "BlockStatement")) {
                captured = captured.concat(exports.extractUndeclaredVars(arg.body));
            }
            else {
                walk(arg, captured, declared);
            }
        });
    }
    else if (["Identifier"].indexOf(tree.type) > -1) {
        if ((captured.indexOf(tree.name) === -1) &&
            (declared.indexOf(tree.name) === -1) &&
            (exports.specialKeywords.indexOf(tree.name) === -1)) {
            captured.push(tree.name);
        }
    }
    else if (["MemberExpression"].indexOf(tree.type) > -1) {
        if (tree.object.name) {
            if ((captured.indexOf(tree.object.name) === -1) && (declared.indexOf(tree.name) === -1)) {
                captured.push(tree.object.name);
            }
        }
        else {
            walk(tree.object.object, captured, declared);
        }
    }
};
exports.extractUndeclaredVars = (ast) => {
    let unknownVars = [];
    const vars = [];
    ast.body.forEach((elm) => {
        if (elm.type === "ExpressionStatement") {
            walk(elm.expression, unknownVars, vars);
        }
        else if (elm.type === "VariableDeclaration") {
            elm.declarations.forEach((dec) => {
                const name = dec.id.name;
                vars.push(name);
            });
        }
        else if ((elm.type === "FunctionDeclaration")) {
            unknownVars = unknownVars.concat(exports.extractUndeclaredVars(elm.body));
        }
    });
    return unknownVars;
};
exports.extractUndeclaredVarsFromCode = (code) => {
    const scopes = detect(code);
    return scopes
        .filter((scope) => [
        'console',
        'space',
        'Date',
        'BigDecimal',
        'window',
        'document',
        'theResult',
        'theResultIs',
        'rule',
        'Math'
    ].indexOf(scope.name) === -1)
        .map((scope) => {
        return scope.name;
    });
};
exports.formatCode = (code) => {
    const ast = Parser.parse(code);
    return recast.prettyPrint(ast, { tabWidth: 2 }).code;
};
exports.getValueForPath = (root, paths) => {
    let object = root;
    if (paths.length !== 0) {
        let indexOfArray = paths.indexOf("?");
        indexOfArray = indexOfArray === -1 ? paths.length : indexOfArray;
        const normalPaths = indexOfArray === paths.length ? paths : paths.slice(0, indexOfArray);
        const remainderPaths = indexOfArray === paths.length ? [] : paths.slice(paths.indexOf("?") + 1);
        normalPaths.forEach((path) => {
            if (object !== undefined) {
                object = object[path];
            }
        });
        if (object !== undefined) {
            if (indexOfArray < paths.length) {
                const arr = object;
                const vals = [];
                arr.forEach((item) => {
                    const arrValue = exports.getValueForPath(item, remainderPaths);
                    if (arrValue !== undefined) {
                        vals.push(arrValue);
                    }
                });
                object = vals.length === 0 ? undefined : vals;
            }
        }
        return object;
    }
    return undefined;
};
function guessTypeOf(value) {
    const type = typeof value;
    if (value && (typeof value.getMonth === "function")) {
        return "date";
    }
    else if (type === "object") {
        return Array.isArray(value) ? "array" : "object";
    }
    else if ((type === "symbol") || (type === "undefined") || (type === "function")) {
        throw `Cannot guess type of ${type}`;
    }
    else if (type === "string") {
        if (((value.indexOf('-') > -1) || (value.indexOf('/') > -1)) &&
            !isNaN(Date.parse(value))) {
            return "date";
        }
        else {
            return "string";
        }
    }
    else {
        return type;
    }
}
exports.guessTypeOf = guessTypeOf;
//# sourceMappingURL=index.js.map